@@ -32,6 +32,12 @@ module CollabQA
         extract_line_with_context(RUBY_EXTCONF_SEARCH_RULE)
       elsif @reasons.include?('OBSOLETE_CODE')
         extract_line_with_context(OBSOLETE_CODE_SEARCH_RULE)
+      elsif @reasons.include?('PATCH_FAILED')
+        extract_line_with_given_context(
+          PATCH_FAILED_SEARCH_RULE,
+          RE2('^Unpack source'),
+          RE2('^FAILED \[dpkg-source died\]')
+        )
       elsif @reasons.include?('CONFIGURE_ERROR')
         extract_log_configure
         # Remove ld/gcc reasons to avoid unrelated config.log errors.
@@ -48,6 +54,8 @@ module CollabQA
         extract_log_timeout
       elsif @reasons.include?('RUBY_TEST_FAILED')
         extract_ruby_test
+      elsif @reasons.include?('TESTS_FAILURES')
+        extract_line_with_summary(TESTS_FAILURES_SEARCH_RULE, 'Tests failures')
       else
         extract_log_default
       end
@@ -55,13 +63,16 @@ module CollabQA
 
     CLANG_SEARCH_RULE=RE2('(^clang: error: ([^linker command failed]|.*)|^error: (invalid|unknown)| fatal error:.*file not found)')
     GCC_SEARCH_RULE=RE2('(\d|.*cc.*): (internal compiler |fatal )?(E|e)rror: ')
-    
+
     RUBY_TEST_SEARCH_RULE=RE2('ERROR: Test "ruby[0-9.]*" failed. Exiting.')
     RUBY_EXTCONF_SEARCH_RULE=RE2('extconf failed: \(Gem::InstallError\)')
-    
-    OBSOLETE_CODE_SEARCH_RULE=RE2('^ERROR: .* is obsolete and no longer supported. .*')
 
+    OBSOLETE_CODE_SEARCH_RULE=RE2('^ERROR: .* is obsolete and no longer supported. .*')
     TIMEOUT_SEARCH_RULE=RE2('Build killed with signal .* after .* minutes of inactivity')
+    PATCH_FAILED_SEARCH_RULE=RE2('^dpkg-source: error: expected .* diff .*')
+
+    #TESTS_FAILURES_SEARCH_RULE=RE2('recipe for target \'override_dh_auto_test\' failed')
+    TESTS_FAILURES_SEARCH_RULE=RE2('recipe for target \'[A-Za-z_]*test[A-Za-z_]*\' failed')
 
     REASONS = [
       [ RE2('^Source-dependencies not satisfied; skipping'), 'BUILDDEPS' ],
@@ -85,7 +96,9 @@ module CollabQA
       [ TIMEOUT_SEARCH_RULE, 'BUILD_TIMEOUT' ],
       [ RUBY_TEST_SEARCH_RULE, 'RUBY_TEST_FAILED' ],
       [ RUBY_EXTCONF_SEARCH_RULE, 'RUBY_EXTCONF_FAILED' ],
-      [ OBSOLETE_CODE_SEARCH_RULE, 'OBSOLETE_CODE' ]
+      [ OBSOLETE_CODE_SEARCH_RULE, 'OBSOLETE_CODE' ],
+      [ PATCH_FAILED_SEARCH_RULE, 'PATCH_FAILED'],
+      [ TESTS_FAILURES_SEARCH_RULE, 'TESTS_FAILURES']
     ]
 
     GCC_LINES = [
@@ -483,6 +496,32 @@ module CollabQA
       @sum_1l = @lines[index]
     end
     
+    def extract_line_with_summary(rule, summary)
+
+      extract_line_with_context(rule)
+      @sum_1l = summary
+    end
+    
+    def extract_line_with_given_context(rule, beg_rule, end_rule)
+
+      # find key line
+      index   = @lines.grep_index(rule)[0]
+      raise "WHERE SHOULD I START?" if index == nil
+
+      # find other points
+      beg_idx = @lines.grep_index(beg_rule)[0] if beg_rule
+      end_idx = @lines.grep_index(end_rule)[0] if end_rule
+      
+      # use defaults if not given/found
+      end_idx = index if end_idx == nil
+      beg_idx = end_idx - WANT_LINES if beg_idx == nil
+      beg_idx = 0 if beg_idx < 0
+
+      @extract = @lines[beg_idx..end_idx]
+      @sum_ml = @lines[beg_idx..end_idx]
+      @sum_1l = @lines[index]
+    end
+    
     #
     # Extract log for timeout fails, trying to clean all the dots and other
     # garbage, that get's generated during the build.
@@ -549,6 +588,7 @@ module CollabQA
       RE2('configure: error: '),
       RE2('ERROR: The following new or changed copyright notices discovered:'),
       RE2('dh_makeshlibs: dpkg-gensymbols .* returned exit code'),
+      RE2('dh_makeshlibs: failing due to earlier errors'),
       RE2('error CS\d+:'),
       RE2('dh_install:.*missing files'),
       RE2('java.*cannot find symbol'),
@@ -581,7 +621,7 @@ module CollabQA
         bf = l[0]
       elsif not (l = @lines.grep_index(RE2('^dpkg-source: error: LC_ALL=C patch '))).empty?
         bf = l[0]
-      elsif not (l = @lines.grep_index(RE2('^dpkg-source: error: expected .* diff .*'))).empty?
+      elsif not (l = @lines.grep_index(RE2(PATCH_FAILED_SEARCH_RULE))).empty?
         bf = l[0]
       else
         raise "WHERE SHOULD I START?"
